/* program Bugs On The Run S - a simpler alife program. */
/* version 1.0 */
/* the world with it's food stays the same as a passive background. */
/* the orgs don't do anything only move and have no properties only */
/* location and direction. */
/* this is a carbon version with a classic evt. loop */
/*																		*/
/* to add var size dyn arrays, just remove comments in create */
/* world and free world, in main enable create world, in */
/* globals declare world[][] as **world */
/* or, instead, declare existing arrays of a constant size which */
/* is larger or equal to WLSIZE and use as much as needed using */
/* WLSIZE. this allowes for var sized worlds without having to recompile. */
/* I tried it on 64x64 arrays with no change in speed, while 128x128 had */
/* a very slight effect on speed (but still faster than dyn arrays). */
/* if you run in turbo speed while prefs dialog is in front with cursor blinking */
/* in a text field, then sim runs about 20% slower. at hyper speed, an open prefs */
/* dialog has no effect. the print events menu also has an impact on performance. */
/* you can click on active area to stop/continue sim. in hyper speed the menus */
/* don't respond so you have to click in the window to stop simulation. */
/* you can also set the speed of the simulation. */
/* other things that affect performance are: show fps menu, and if your other */
/* applications are running at the same time. */
/* if you choose print events from the preferences dialog, then the corresponding */
/* menu item in the menu bar will be checked if it wasn't. same also at startup. */
/* you can drag orgs. to cancel a */
/* drag, continue dragging out of the green world. */
/* to move a selected org by one cell, use the arrow keys while main window is */
/* in front. */
/* instead of redrawing entire array, only changes are drawn, */
/* except when hyper stops or changes speed, or when limiting FPS, or */
/* starting a new simulation, both result in a call to redraw_scr. */
/* if the number of orgs drawn are different from number in text then */
/* it's because one or more orgs are on the same cell (layered). */


/* Copyright (c) 2009 Noam Lenz
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:
   1. Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
   3. The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
   IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
   NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
   THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  */


#include <Carbon/Carbon.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include <OpenGL/glext.h>
#include <AGL/agl.h>

/* Toolbox Constants */
#define kMainNibFileName	 	"Bugs On The Run"
#define kMenuBarNibName		 	"MainMenu"
#define kBugsOnTheRunNibName	"WIND"
#define kMLTENibName				"MLTE"
#define kBaseResID 				128
#define kAppSignature			'BOTR'

#define kAboutDialogID			128
//#define kAboutDialogRfCn		3
#define kIconUserItem			2

//#define kPrefsDialogRfCn		1
#define kPrefsDialogID			129
#define kMaxItersText			3
#define kSlowText					10
#define kFastText					12
#define kTurboText				14
#define kSetStopPtBox			17
#define kStopItersStatText		18
#define kStopItersText			19
#define kINumOrgsText			25
#define kRandINumOrgsBox		26
#define kRerunSavedSimBox		27
#define kRandomizeProgBox		28
#define kHelpButton				31
#define kSetDefaultButton		32
#define kRevertButton			33
#define kApplyButton				34
#define kOKButton					35

#define kMinTextPosition		0
#define kMaxTextPosition		10

#define kAppCFStr				CFSTR("BugsOnTheRun prefs")

#define kReturn				(SInt8) 0x0D  // keys pressed
#define kEnter					(SInt8) 0x03

#define kSliderDialogID			130
#define kSliderDialogRfCn		2
#define kSlider					1
#define kSliderStaticText		8


#define kTicksPerSecond			60

#define kAddCheckMark			true
#define kRemoveCheckMark		false

#define mApple					kBaseResID
#define iAbout					1

#define mFile					kBaseResID+1
#define iNew					1
#define iStop					2
#define iContinue				3
#define iRunLast				4
#define iSave					5
#define iRunSaved				6
//#define iQuit				7

#define mSpeed					kBaseResID+2
#define iSlow					1
#define iFast					2
#define iTurbo					3
#define iHyper					4
#define iStep					5
#define iVariable				6
#define iSetVar				7
#define i60FPS					9  // 8 is a separator line

#define mOutput				kBaseResID+3
#define iShowFPS				1
#define iShowStats			2
#define iPrintEvents			3

#define UPICN					128
#define NUMICNS				13

/* Program Constants */
#define ACTVSTART				10
#define ACTVEND				290
#define MINTIMERDELAY		0.000001 // don't change to 0 or less. 0 - one shot timer
#define MAXTIMERDELAY		10000
#define SLOW					2.0 // delay time in sec.s
#define FAST					1.0
#define TURBO					MINTIMERDELAY
#define VARIABLE				1.0
#define SLIDERVAL				180
#define CELLSIZE				10
#define WLSIZE					28  // size of world array
#define UP						0   // directions of orgs
#define LEFT					1
#define DOWN					2
#define RIGHT					3  // 4 through 11 are the directions in 2 more colors
#define FOOD					12
#define GROUND					13

/*  simulation parameters  */
/* (parameters in prefs file overide the following ones) */
#define MAXITERS				10000000 // max. number of iterations
#define SETSTOPPT				false	// enable or disable the following (stop point)
#define STOPITERS				2000 // stop tmp at number of iterations
#define INUMORGS				4   // initial number of orgs
#define INITFOOD				4   // initial food amount - more is less
#define MUTRATE				10  // mutation rate - more is less
#define RANDINUMORGS			false	// if true: starts appl. with rand. num. of orgs 
										//   between 1 and INUMORGS
#define RERUNSAVEDSIM		false	// will cause all new sims to repeat the saved one
#define SAVEDSEED				1	// the deflt seed for myrand when there's no prefs file
#define RANDOMIZEPROG		false	// 0 or 1.if 0: goes through same sequence of sims

/* max & min numorgs isn't always exact since more than 1 org can be born or die in */
/* a single iteration. (e.g if you set maxnumorgs = 30 and in a certain iteration */
/* norgs==29 and in the next iteration 2 orgs are born then norgs jumps to 31 and */
/* there it stops, not at 30. */
/* the difference between run saved menu and rerun saved sim pref is that the menu runs */
/* the saved sim only once when chosen, while the pref causes it to be rerun every time */
/* you choose new, run last, or run saved menus. */
									
typedef struct org{
	int x;
	int y;
	int direc;
} org;

/* Global Variables */
CFBundleRef gBundle;
IBNibRef gNibs;
int selcx, selcy; // on which coordinates was an org selected
WindowRef window;  // main graphics window
//WindowRef dialog = NULL;
//DialogRef prefsDialog = NULL, aboutDialog = NULL, sliderDialog = NULL;
//WindowRef helpPrefsWindow = NULL;
MenuHandle	Amenu, Fmenu, Smenu, Omenu;
EventLoopTimerRef theTimer;
EventLoopTimerUPP myTimer;
int simNum = 0;
int seed = 1;  // seed of myrand func
unsigned long int lastSeed = 0, savedSeed;
ControlActionUPP	gSliderActionUPP;
SInt16 sliderVal;
ControlRef sliderStaticTextCR;
//long 	gFillColor = greenColor;  // from 8 color classic QuickDraw; RGB 0,128,17
int world[WLSIZE][WLSIZE]; // set to 64x64, use as needed
org orgs[WLSIZE * WLSIZE];
int	gDone, stop, cont_enabled; /* stop==1 demo isn't running, */
/* c_e==1 you can choose cont..At beginning before running and also */
/* at end - stop==1 but c_e==0 */
int prev_step_stop = 0; // stop state before a series of steps
int end_sim = 0;  // used for not allowing to step after end of sim
short speed = iSlow; 
int selectOrgIndex = -1;
int prinfo = 0; // if prinfo==1 info will be printed about an org
//int printed_stats = 0;
/* prefs globals: #defines (=default and 1st run of app) are set in getPrefs. */ 
double Slow, Fast, Turbo, Variable;
int maxIters, maxGens, maxOrgs;
Boolean setStopPt;
int stopIters, stopGens, stopOrgs;
int iNumOrgs;
Boolean randINumOrgs, randomizeProg; // randomizeProg is applied after relaunching appl
Boolean rerunSavedSim; // repeats saved sim when using new, run last or run saved menus
Boolean prnEvents, prnEventsPref; // if prnEventsPref is true then prnEvents=true at startup
Boolean prnStatsAtEnd, prnFPS = false;  
double delay = SLOW; // delay time in seconds. is inited in init_globals
int iters = 0, greatestGen = 1, norgs = 0;
int diedage = 0, diedenloss = 0, reprod = 0;  // for statistics
int total = 0;  // a measure of orgs processed (in orgs*iters units)
int last_iters = 0;  // for use in measuring FPS/CPS
int limit_to_60FPS = 0;  // if the 60 FPS menu is selected
int limited_FPS = 0;  // if limit_to_60FPS and delay < 0.016667 secs (if running faster than 60FPS)
Boolean	gInBackground;		// maintained by Initialize and DoEvent for mlte

AGLContext  myAGLContext = NULL;
CGContextRef myBitmapContext;
CGContextRef myContext;
CGImageRef icon[NUMICNS];
CGImageRef myImage;
GLuint texture[NUMICNS];
GLuint fontList;
	
/* Prototypes */
OSErr quitAppleEventHandler(AppleEvent *, AppleEvent *, SInt32);
OSErr prefsEventHandler(AppleEvent *, AppleEvent *, SInt32);
Boolean Init(void);
GLuint buildFontGL (AGLContext ctx, GLint fontID, Style face, GLint size);
Boolean MenuBarInit(void);
void WindowInit(void);
void setup_gl(void);
void CreateBitmapImages(void);
void CreateTextures(int pixelsWide, int pixelsHigh);
void doIdle(void);
void EventLoop(void);
void DoEvent(EventRecord *eventPtr);
void HandleMouseDown(EventRecord *eventPtr);
void HandleMenuChoice(long menuChoice);
//void HandleKeyDown(EventRecord *eventPtr);
void HandleAppleChoice(short item);
OSStatus AboutWindowEventHandler(EventHandlerCallRef myHandler, EventRef event,
						void *userData);
void HandleFileChoice(short item);
void HandleSpeedChoice(short item);
void HandleOutputChoice(short item);
void HandleSliderDialog(void);
OSStatus SliderWindowEventHandler(EventHandlerCallRef myHandler, EventRef event,
						void *userData);
void sliderActionFunction(ControlRef controlRef, ControlPartCode partCode);
void HandlePrefsDialog(void);
OSStatus PrefsWindowEventHandler(EventHandlerCallRef myHandler, EventRef event,
											void *userData);
int getIntVal(WindowRef dialog, int textItem);
double getDoubleVal(WindowRef dialog, int textItem);
void SetPrefsDialogVals(WindowRef dialog, Boolean defaultVal);
void HandleHelpPrefs(void);
OSStatus HelpPrefsWindowEventHandler(EventHandlerCallRef myHandler, EventRef event, void *userData);
void HandleInContent(EventRecord *eventPtr, Point globalPoint, WindowRef theWindow);
void DrawOrgPrecisely(int dir, int x, int y);
void HandleDraw(void);
void DrawIcon(int icn, int x, int y);
void DrawGreen(int x, int y);
void MoveOrgByArrowKeys(SInt8 charCode);
void draw_frame(int x, int y, int col);
void drawCStringGL (char * cstrOut);
void outext(void);
void outext2(void);
void outext3(int x, int y, int col);
void outext4(double fps);
void clr_selorg_text(void);
void clr_dragged_org_text(void);
void clr_FPS_text(void);
void calcFPS(void);
int timed_out_60FPS(void);
void redraw_scr(void);
void getPrefs(void);
void savePrefs(void);
void saveRandSeed(void);
void init_world(void);
void new_sim(void);
void rand_dat_org(int orgIndex);
void reproduce(int orgIndex);//, int *greatestGen);
void die(int orgIndex);
void clone_org(int orgIndex);
int find_org(int x, int y);
//void free_world(void);
int myrand(void);
void wait_mess(void);
void waitime(void);
static UInt32 GetSleep(void);


int main(void)
{  	
	
	Init();
	MenuBarInit();
	WindowInit();
	getPrefs();
	setup_gl();
	CreateBitmapImages();
	CreateTextures(CELLSIZE, CELLSIZE);
	/*if ((orgs = (org *) calloc(2 * maxOrgs, sizeof(org))) == NULL) {
     	 printf("Error: calloc failed for org array\n");
      	exit(1);
   	}*/
	//create_world();
	myTimer = NewEventLoopTimerUPP((EventLoopTimerProcPtr) doIdle);
	InstallEventLoopTimer(GetMainEventLoop(), Slow, Slow, myTimer, NULL, &theTimer);
	EventLoop();
	//free(orgs);
	//free_world();
	
	if (gNibs != NULL)
		DisposeNibReference(gNibs);
	
	if (gBundle != NULL)
		CFRelease(gBundle);
	//Terminate();
	
	return 0;
}

OSErr quitAppleEventHandler(AppleEvent *appleEvt,
								AppleEvent *reply, SInt32 refcon)
{	
	//#pragma unused (appleEvt, reply, refcon)
	savePrefs();
	gDone = 1;
	return false;
}

OSErr prefsEventHandler(AppleEvent *appleEvt, AppleEvent *reply, SInt32 refcon)
{
	//#pragma unused (appleEvt, reply, refcon)
	OSErr err;
	DescType	returnedType;
	Size		actualSize;

	err = AEGetAttributePtr(appleEvt, keyMissedKeywordAttr, typeWildCard, &returnedType,
					NULL, 0, &actualSize);

	if (err == errAEDescNotFound)
	{
		HandlePrefsDialog();
		err = noErr;
	} 
	else if (err == noErr)
		err = errAEParamMissed;

	return err;
}

Boolean Init(void)
{

	OSErr err;
	Boolean isInitialized;
	
	isInitialized = false;
	gBundle = NULL;
	gNibs = NULL;
	
	gBundle = CFBundleGetMainBundle();
	
	if (gBundle != NULL)
	{
		err = CreateNibReferenceWithCFBundle(gBundle, CFSTR(kMainNibFileName), &gNibs);
		
		if ((err == noErr) && (gNibs != NULL))
			isInitialized = true;
	}
	FlushEvents(everyEvent,0);
	err = AEInstallEventHandler(kCoreEventClass, kAEQuitApplication,
			NewAEEventHandlerUPP((AEEventHandlerProcPtr) quitAppleEventHandler), 0, false);
	if (err != noErr)
		ExitToShell();
	err = AEInstallEventHandler(kCoreEventClass, kAEShowPreferences,
			NewAEEventHandlerUPP((AEEventHandlerProcPtr) prefsEventHandler), 0, false);
	if (err != noErr)
		ExitToShell();
	/*AEInstallEventHandler(kCoreEventClass, kAEOpenApplication,
			NewAEEventHandlerUPP(HandleOapp), 0, false);
	if (err != noErr)
		ExitToShell();*/
    gInBackground = false;	
	return isInitialized;
}


// AGL bitmap font setup for the info on the main graphics window
GLuint buildFontGL (AGLContext ctx, GLint fontID, Style face, GLint size)
{
	GLuint listBase = glGenLists (256);
	if (aglUseFont (ctx, fontID , face, size, 0, 256, (long) listBase)) {
		glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
		return listBase;
	} else {
		glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
		glDeleteLists (listBase, 256);
		return 0;
	}
}

Boolean MenuBarInit(void)
{
	OSErr err;
	Boolean isMenuInstalled;
	
	isMenuInstalled = false;
	
	err = SetMenuBarFromNib(gNibs, CFSTR(kMenuBarNibName));
	
	if (err == noErr)
		isMenuInstalled = true;
	
	Amenu = GetMenuHandle(mApple);
	Fmenu = GetMenuHandle(mFile);
	Smenu = GetMenuHandle(mSpeed);
	Omenu = GetMenuHandle(mOutput);
	CheckMenuItem(Smenu, iSlow, kAddCheckMark);
	
	EnableMenuCommand(NULL,'pref');
		
	return isMenuInstalled;
}

void WindowInit(void)
{
	OSErr theErr;
	
	theErr = CreateWindowFromNib(gNibs, CFSTR(kBugsOnTheRunNibName), &window);
	
	if ((theErr == noErr) && (window != NULL))
		ShowWindow(window);
	
	SetPort(GetWindowPort(window));	/* set window to current graf port */
}

void setup_gl(void) {
	Rect rectPort;
	GLint attributes[] =  {AGL_RGBA, AGL_DOUBLEBUFFER, AGL_ACCELERATED, AGL_DEPTH_SIZE, 1, AGL_NONE};
	int x,y;
	short fNum;
	
	AGLPixelFormat myAGLPixelFormat;
	
	myAGLPixelFormat = aglChoosePixelFormat (NULL, 0, attributes);
	myAGLContext = aglCreateContext (myAGLPixelFormat, NULL);
	aglDestroyPixelFormat(myAGLPixelFormat);
	aglSetDrawable (myAGLContext, GetWindowPort (window));
	aglSetCurrentContext (myAGLContext);
	GetWindowPortBounds (window, &rectPort);
	glViewport (0, 0, rectPort.right - rectPort.left, rectPort.bottom - rectPort.top);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	// set orthograhic 1:1  pixel transform in local view coords
	gluOrtho2D(0, 440, 300, 0);
	glDisable(GL_DEPTH_TEST);  // no Z coordinates since 2D only
	glDisable(GL_BLEND);
	glMatrixMode(GL_MODELVIEW);
	
	GetFNum ("\pGeneva", &fNum); // build font
	fontList = buildFontGL (myAGLContext, fNum, normal, 10);
	
	
	/* Clear the screen */
	glClearColor(1.0, 1.0, 1.0, 1.0);
	glClear(GL_COLOR_BUFFER_BIT);
}

void CreateBitmapImages(void)
{
	CGDataProviderRef provider;
	CFStringRef path;
	CFURLRef url;
	char filenumstr[3];
	int iconnum;
	
	for ( iconnum = 0; iconnum < NUMICNS; iconnum++) {
		sprintf(filenumstr, "%d", iconnum + UPICN);
		url = CFBundleCopyResourceURL(gBundle, CFStringCreateWithCString(NULL,filenumstr, kCFStringEncodingASCII), CFSTR("png"), NULL);
		provider = CGDataProviderCreateWithURL(url);
		CFRelease(url);
		icon[iconnum] = CGImageCreateWithPNGDataProvider(provider, NULL, false, kCGRenderingIntentDefault);
		CGDataProviderRelease(provider);
	}
}

void CreateTextures(int pixelsWide, int pixelsHigh)
{
	CGContextRef    context = NULL;
	CGColorSpaceRef colorSpace;
	void *          bitmapData;
	int             bitmapByteCount;
	int             bitmapBytesPerRow;
	CGRect rect = {{0, 0}, {pixelsWide, pixelsHigh}};
	int k;
	
	bitmapBytesPerRow   = (pixelsWide * 4);
	bitmapByteCount     = (bitmapBytesPerRow * pixelsHigh);
	
	colorSpace = CGColorSpaceCreateDeviceRGB();
	bitmapData = malloc(bitmapByteCount);
	context = CGBitmapContextCreate (bitmapData, pixelsWide, pixelsHigh, 8, bitmapBytesPerRow, colorSpace, kCGImageAlphaPremultipliedLast);
	
	glEnable(GL_TEXTURE_RECTANGLE_EXT);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);  // decal means just paste our textures as-is
	glGenTextures(NUMICNS, &texture[0]);
	for (k = 0; k < NUMICNS; k++) {
		CGContextDrawImage(context, rect, icon[k]);
		glBindTexture(GL_TEXTURE_RECTANGLE_EXT, texture[k]);
		glTexImage2D(GL_TEXTURE_RECTANGLE_EXT, 0, GL_RGBA, pixelsWide, pixelsHigh,
					 0, GL_RGBA, GL_UNSIGNED_BYTE, bitmapData);
	}
		
	CGContextRelease(context);
	CGColorSpaceRelease( colorSpace );
	
	free (bitmapData);
	
}

void doIdle(void)
{	
	static int prev_stop = 0;
	
	if (prev_stop != stop) {  // so that while idle, doesn't overload cpu
		RemoveEventLoopTimer(theTimer);
		if (stop == 1)
			InstallEventLoopTimer(GetMainEventLoop(), 0, 0.1, myTimer, NULL, &theTimer);
		else
			InstallEventLoopTimer(GetMainEventLoop(), delay, delay, myTimer, NULL, &theTimer);
		prev_stop = stop;
	}
	if (prnFPS)
		calcFPS();
	while (stop == 0 && (speed == iHyper && !Button()) && !gDone) {
			HandleDraw();  // for hyper speed
			if (prnFPS && iters % 400 == 0)
				calcFPS();
	}
	if (stop == 0 && speed != iHyper)
		HandleDraw();
}

void EventLoop(void)
{
	EventRecord event;
	Boolean gotEvent;
	
	if (randomizeProg)
		seed = time(NULL);
	if (prnEvents)
		CheckMenuItem(Omenu, iPrintEvents, kAddCheckMark);
	new_sim();
	gDone = 0;
	while (gDone == 0) {
		gotEvent = WaitNextEvent(everyEvent, &event, GetSleep(), nil);  // kSleep, nil
			DoEvent(&event);
	}
}

void DoEvent(EventRecord *eventPtr)
{
	WindowRef windw;
	GrafPtr oldPort;
	SInt8 charCode;
	Boolean isDialogEvent;  // mlte
	
	isDialogEvent = IsDialogEvent(eventPtr);
	if (isDialogEvent)
	{
		DialogPtr	theDialog = NULL;
		short		itemHit;
                
		DialogSelect(eventPtr, &theDialog, &itemHit);
	}
	switch (eventPtr->what) {
		case nullEvent:
			break;
		case mouseDown:
			HandleMouseDown(eventPtr);
			break;
		case autoKey:
		case keyDown:
			if ((eventPtr->modifiers & cmdKey) == 0) {
				charCode = eventPtr->message & charCodeMask;
				if (selectOrgIndex > -1 && (charCode == kLeftArrowCharCode ||
						charCode == kRightArrowCharCode || charCode == kUpArrowCharCode ||
						charCode == kDownArrowCharCode))
					MoveOrgByArrowKeys(charCode);
			}
			else
				HandleMenuChoice(MenuEvent(eventPtr));
			break;
		case updateEvt:
			windw = (WindowRef)eventPtr->message;
			if (windw==window) {
				GetPort(&oldPort);
				SetPort(GetWindowPort(windw));
				BeginUpdate(windw);
				redraw_scr();
				outext();
				if (prinfo == 1) {
					outext2();
					selcx = orgs[selectOrgIndex].x;
					selcy = orgs[selectOrgIndex].y;
					draw_frame(selcx, selcy, 1);
				}
				EndUpdate(windw);
				SetPort(oldPort);
			}
			break;
		case kHighLevelEvent:
			AEProcessAppleEvent(eventPtr);
			break;
	}
}

void HandleMouseDown(EventRecord *eventPtr)
{
	WindowRef	whichWindow;
	short			thePart;
	long			menuChoice;
	TXNObject	object = NULL;
	
	thePart = FindWindow(eventPtr->where, &whichWindow);
	switch (thePart) {
		case inMenuBar:
			menuChoice = MenuSelect(eventPtr->where);
			HandleMenuChoice(menuChoice);
			break;
		case inContent:
			if (whichWindow != FrontWindow())
				SelectWindow(whichWindow);
			if (whichWindow == window)
				HandleInContent(eventPtr, eventPtr->where, whichWindow);
			break;
		case inDrag:
			//if (whichWindow == window)
			DragWindow(whichWindow, eventPtr->where, NULL);
			break; 
	}
}


//	Calculate sleep value for WaitNextEvent.
static UInt32 GetSleep()
{
	UInt32		sleep;
	WindowPtr	windw;
	
	sleep = GetCaretTime();				// blink time for the insertion point
	return sleep;
}

void HandleMenuChoice(long menuChoice)
{
	short menu;
	short item;
	
	if (menuChoice != 0) {
		menu = HiWord(menuChoice);
		item = LoWord(menuChoice);
		switch (menu) {
			case mApple:
				HandleAppleChoice(item);
				break;
			case mFile:
				HandleFileChoice(item);
				break;
			case mSpeed:
				HandleSpeedChoice(item);
				break;
			case mOutput:
				HandleOutputChoice(item);
				break;
		}
		HiliteMenu(0);
	}
}

void HandleAppleChoice(short item)
{
	static WindowRef aboutDlg = NULL;
	EventHandlerUPP	dialogUPP;
	OSStatus	err = noErr;
	EventTypeSpec 	dialogSpec[] = { {kEventClassKeyboard, kEventRawKeyDown},
											  {kEventClassWindow, kEventWindowClose} };
	ControlRef					ctl;
	Rect						bounds;
	ControlImageContentInfo		content = { kControlContentIconRef };
	ProcessSerialNumber			psn = { 0, kCurrentProcess };
	FSRef						appLocation;
	const ControlID				kIconCtlID = { 'ICON', 0 };
	
	switch (item) {
		case iAbout:
			if (aboutDlg == NULL) {
				err = CreateWindowFromNib(gNibs, CFSTR("About"), &aboutDlg);
				if (err != noErr)
					return;
				dialogUPP =  NewEventHandlerUPP(AboutWindowEventHandler);
			    err = InstallWindowEventHandler(aboutDlg, dialogUPP, GetEventTypeCount(dialogSpec),
			    		dialogSpec, (void *) aboutDlg, NULL);
				GetProcessBundleLocation( &psn, &appLocation );
				GetIconRefFromFileInfo(&appLocation, 0, NULL, 0, NULL,
									kIconServicesNormalUsageFlag, &content.u.iconRef, NULL);
				GetControlByID(aboutDlg, &kIconCtlID, &ctl);
				GetControlBounds(ctl, &bounds);
				DisposeControl(ctl);
				
				CreateIconControl( aboutDlg, &bounds, &content, true, &ctl );
				ShowWindow(aboutDlg);
		   	}
		   	else {
		   		ShowWindow(aboutDlg);
		   		SelectWindow(aboutDlg);
		   	}
		   	DisableMenuItem(Amenu, iAbout);
			break;
		default:
			break;
	}
}

OSStatus AboutWindowEventHandler(EventHandlerCallRef myHandler, EventRef event,
						void *userData)
{
	//#pragma unused (myHandler)
    OSStatus 	result = eventNotHandledErr;
    UInt32		eventClass, eventKind;
    WindowRef	dialog	= (WindowRef) userData;
    EventRecord	evtRec;  
    SInt8		charCode;
    
    eventClass = GetEventClass(event);
    eventKind = GetEventKind(event);
    
	if (eventClass == kEventClassWindow && eventKind == kEventWindowClose) {
		HideWindow(dialog);
		EnableMenuItem(Amenu, iAbout);
		result = noErr;
	}
	else if (eventClass == kEventClassKeyboard && eventKind == kEventRawKeyDown) {
		ConvertEventRefToEventRecord(event, &evtRec);
		charCode = evtRec.message & charCodeMask;
		if (charCode == kEscapeCharCode) {
			HideWindow(dialog);
			EnableMenuItem(Amenu, iAbout);
			result = noErr;
		}
	}
    return result;
}

void HandleFileChoice(short item)
{
	
	switch (item) {
		case iNew:
			seed = lastSeed + 1;
			new_sim();
			break;
		case iStop:
			if (stop == 0) {
				stop = 1;
				cont_enabled = 1;
				DisableMenuItem(Fmenu, iStop);
				EnableMenuItem(Fmenu, iContinue);
				if (speed == iHyper || limited_FPS == 1) {
					redraw_scr();
					outext();
				}
			}
			break;
		case iContinue:
			if (stop == 1 && cont_enabled == 1) {
				if (speed == iStep && end_sim == 0) {
					HandleDraw();
					stop = 1;
				}
				else {
					stop = 0;
					cont_enabled = 0;
					DisableMenuItem(Fmenu, iContinue);
					EnableMenuItem(Fmenu, iStop);
				}
			}
			break;
		case iRunLast:
			seed = lastSeed;
			new_sim();
			break;
		case iSave:
			savedSeed = lastSeed;
			saveRandSeed();
			break;
		case iRunSaved:
			seed = savedSeed;
			new_sim();
			break;
		/*case iQuit:
			gDone = 1;
			break;*/
	}
}

void HandleSpeedChoice(short item)
{
	short last_item = speed;
	
	if (item != last_item) {
		if (item == iSlow)
			delay = Slow;
		else if (item == iFast)
			delay = Fast;
		else if (item == iTurbo)
			delay = Turbo;
		else if (item == iHyper) // doesn't affect speed since hyper doesn't
			delay = 0.1;   // use timer + when idle doesn't overload cpu
		else if (item == iVariable)
			delay = Variable;
		
		if (last_item == iStep && item != i60FPS) {
			stop = prev_step_stop;
			if (stop == 0) {
				cont_enabled = 0;
				EnableMenuItem(Fmenu, iStop);
				DisableMenuItem(Fmenu, iContinue);
			}
		}
		
		if (item == iStep) // get stop state from previous speed
			prev_step_stop = stop;
		
		if (item == iSetVar)  // set variable
			HandleSliderDialog();
		
		if (item == i60FPS) {
			limit_to_60FPS = !limit_to_60FPS;
			if (limit_to_60FPS)
				CheckMenuItem(Smenu, item, kAddCheckMark);
			else
				CheckMenuItem(Smenu, item, kRemoveCheckMark);
		}
		
		if (stop == 1) {  // stop == 0 handled in handledraw. we need this because when
			if (limit_to_60FPS && delay < 0.016667)  // stop == 1 then handledraw isn't called
				limited_FPS = 1;
			else
				limited_FPS = 0;
		}
		
		if (item == i60FPS || item == iSetVar) // to skip setting: timer (handled in handlesliderdlg
		// or in item == i60FPS above), speed = item  (to reenter surrounding ifs)  so that funcs
		// can be recalled to show hidden slider + check 60FPS, and checkmenuitems 
			return;
		
		if (stop == 0) {
			RemoveEventLoopTimer(theTimer);
			InstallEventLoopTimer(GetMainEventLoop(), 0, delay, myTimer, NULL, &theTimer);
		}
		speed = item;
		if ((last_item == iHyper || limited_FPS == 1) && stop == 0) {
			redraw_scr();
			outext();
			if (prinfo == 1) {
				outext2();
				selcx = orgs[selectOrgIndex].x;
				selcy = orgs[selectOrgIndex].y;
				draw_frame(selcx, selcy, 1);
			}
		}
		CheckMenuItem(Smenu, item, kAddCheckMark);
		CheckMenuItem(Smenu, last_item, kRemoveCheckMark);
	}
	if (item == iStep && end_sim == 0) {
		if (cont_enabled == 0) {
			cont_enabled = 1;
			EnableMenuItem(Fmenu, iContinue);
			DisableMenuItem(Fmenu, iStop);
		}
		HandleDraw();
		stop = 1;
	}
} 

void HandleOutputChoice(short item)
{
	switch (item) {
		case iShowFPS:
			if (prnFPS == false) {
				prnFPS = true;
				CheckMenuItem(Omenu, item, kAddCheckMark);
			}
			else {
				prnFPS = false;
				CheckMenuItem(Omenu, item, kRemoveCheckMark);
				clr_FPS_text();
			}
			break;
	}
}

void HandleSliderDialog(void)
{
	//DialogRef sliderDialog;
	static WindowRef sliderDlg = NULL;
	EventHandlerUPP	dialogUPP;
	OSStatus	err = noErr;
	EventTypeSpec 	dialogSpec[] = { //{kEventClassControl, kEventControlHit},
									 {kEventClassKeyboard, kEventRawKeyDown},
									 {kEventClassWindow, kEventWindowClose} };
	ControlID	cID;
	ControlRef controlRef;
	CFStringRef  text;
	
	if (sliderDlg == NULL) {  // create new dialog
		err = CreateWindowFromNib(gNibs, CFSTR("Slider"), &sliderDlg);
		if (err != noErr)
			return;
		cID.signature = 'BOTR';
		cID.id = kSlider;
		GetControlByID(sliderDlg, &cID, &controlRef);
		SetControlValue(controlRef, sliderVal);
		cID.id = kSliderStaticText;
		GetControlByID(sliderDlg, &cID, &sliderStaticTextCR);
		text = CFStringCreateWithFormat(NULL, NULL, CFSTR("%.2g"), Variable);
		SetControlData(sliderStaticTextCR, 0, kControlStaticTextCFStringTag,
							sizeof(CFStringRef), &text);
		
		gSliderActionUPP = NewControlActionUPP((ControlActionProcPtr) 
													sliderActionFunction);
		SetControlAction(controlRef, gSliderActionUPP);
		dialogUPP =  NewEventHandlerUPP(SliderWindowEventHandler);  // re-use existing one
		err = InstallWindowEventHandler(sliderDlg, dialogUPP, GetEventTypeCount(dialogSpec),
			    dialogSpec, (void *) sliderDlg, NULL);
	   	ShowWindow(sliderDlg);
	}
	else {
		ShowWindow(sliderDlg);
		SelectWindow(sliderDlg);
	}
	delay = Variable;
	if (speed != iVariable) {
		CheckMenuItem(Smenu, iVariable, kAddCheckMark);
		CheckMenuItem(Smenu, speed, kRemoveCheckMark);
		speed = iVariable;
		if (stop == 0) {
			RemoveEventLoopTimer(theTimer);
			InstallEventLoopTimer(GetMainEventLoop(), 0, delay, myTimer, NULL, &theTimer);
		}
	}
}

OSStatus SliderWindowEventHandler(EventHandlerCallRef myHandler, EventRef event,
						void *userData)
{
	//#pragma unused (myHandler)
    OSStatus 	result = eventNotHandledErr;
    UInt32		eventClass, eventKind;
    WindowRef	dialog	= (WindowRef) userData;
    EventRecord	evtRec;  
    SInt8		charCode;
    
    eventClass = GetEventClass(event);
    eventKind = GetEventKind(event);
    
	if (eventClass == kEventClassWindow && eventKind == kEventWindowClose) {
		HideWindow(dialog);
		EnableMenuItem(Smenu, iSetVar);
		result = noErr;
	}
	else if (eventClass == kEventClassKeyboard && eventKind == kEventRawKeyDown) {
		ConvertEventRefToEventRecord(event, &evtRec);
		charCode = evtRec.message & charCodeMask;
		if (charCode == kEscapeCharCode) {
			HideWindow(dialog);
			EnableMenuItem(Smenu, iSetVar);
			result = noErr;
		}
	}
	/*else if (eventClass == kEventClassControl && eventKind == kEventControlHit) {
		Variable = delay;
		if (speed != iVariable) {
			CheckMenuItem(Smenu, iVariable, kAddCheckMark);
			CheckMenuItem(Smenu, speed, kRemoveCheckMark); 
			speed = iVariable;
			result = noErr;
		}
	}*/
	

    return result;
}

void sliderActionFunction(ControlRef controlRef, ControlPartCode partCode)
{
	//#pragma unused (partCode)
	
	double dval;
	CFStringRef  text;
	
	sliderVal = GetControlValue(controlRef);
	dval = (2000 - sliderVal) / 400.0;
	delay = 1.25e-6 * pow(20.0, dval);
	if (stop == 0) {
		RemoveEventLoopTimer(theTimer);
		InstallEventLoopTimer(GetMainEventLoop(), delay/2, delay, myTimer, NULL, &theTimer);
	}
	if (stop == 1) {  // stop == 0 handled in handledraw. we need this because when
		if (limit_to_60FPS && delay < 0.016667)  // stop == 1 then handledraw isn't called
			limited_FPS = 1;
		else
			limited_FPS = 0;
	}
	text = CFStringCreateWithFormat(NULL, NULL, CFSTR("%.2g"), delay);
	SetControlData(sliderStaticTextCR, 0, kControlStaticTextCFStringTag,
						sizeof(CFStringRef), &text);
	Draw1Control(sliderStaticTextCR);
	Variable = delay;
	if (speed != iVariable) {
		CheckMenuItem(Smenu, iVariable, kAddCheckMark);
		CheckMenuItem(Smenu, speed, kRemoveCheckMark);
		if (speed == iStep) {
			stop = prev_step_stop;
			if (stop == 0) {
				cont_enabled = 0;
				EnableMenuItem(Fmenu, iStop);
				DisableMenuItem(Fmenu, iContinue);
			}
		}
		speed = iVariable;
	}
}

void HandlePrefsDialog(void)
{
	EventTypeSpec 	dialogSpec[] = { {kEventClassControl, kEventControlHit},
									 {kEventClassKeyboard, kEventRawKeyDown},
									 {kEventClassKeyboard, kEventRawKeyRepeat},
									 {kEventClassWindow, kEventWindowClose} };
	EventHandlerUPP	dialogUPP;
	static WindowRef 	dialog = NULL;
    OSStatus	err = noErr;
	ControlID	cID;
	ControlRef	controlRef;
	ControlEditTextSelectionRec	selRec;
	Size		txtSize, slSize;
	
	if (dialog == NULL) {
		err = CreateWindowFromNib(gNibs, CFSTR("Preferences"), &dialog);
		if (err != noErr)
			return;
		dialogUPP =  NewEventHandlerUPP(PrefsWindowEventHandler);
	    err = InstallWindowEventHandler(dialog, dialogUPP, GetEventTypeCount(dialogSpec),
	    		dialogSpec, (void *) dialog, NULL);
		SetPrefsDialogVals(dialog, false);  // set dialog's values to current values
		ShowWindow(dialog);
		cID.signature = 'BOTR'; 
		cID.id = kMaxItersText;
		GetControlByID(dialog, &cID, &controlRef);
		SetKeyboardFocus(dialog, controlRef, kControlFocusNextPart);
		GetControlDataSize(controlRef, kControlEditTextPart, kControlEditTextTextTag, &txtSize);
		GetControlData(controlRef, kControlEditTextPart, kControlEditTextSelectionTag,
							sizeof(ControlEditTextSelectionRec), &selRec, &slSize );
		selRec.selStart = 0;
		selRec.selEnd = txtSize;
		SetControlData(controlRef, kControlEntireControl, kControlEditTextSelectionTag,
								sizeof(ControlEditTextSelectionRec), &selRec);
	}
	else {  // show existing dialog
		SetPrefsDialogVals(dialog, false);  // set dialog's values to current values
		ShowWindow(dialog);
		SelectWindow(dialog);
	}
	DisableMenuCommand(NULL,'pref');

    //HideWindow(dialog);
    //DisposeWindow(dialog);
    //DisposeEventHandlerUPP(dialogUPP);
}

OSStatus PrefsWindowEventHandler(EventHandlerCallRef myHandler, EventRef event,
						void *userData)
{
	//#pragma unused (myHandler)
    OSStatus 	result = eventNotHandledErr;
   	ControlRef	controlRef, stopItersStatCRef;// stopGensStatCRef, stopOrgsStatCRef;
	ControlID	controlID, cID;
    UInt32		eventClass, eventKind;
    Boolean		boolVal;
    int			popupVal;
    int			intval;
    double		dblval = 0.0;
	int			orgCnt;
	int			posx, posy, oldmaxorgs;
    WindowRef	dialog	= (WindowRef) userData;
    EventRecord	evtRec;  
    SInt8		charCode;
    
    eventClass = GetEventClass(event);
    eventKind = GetEventKind(event);
    
	if (eventClass == kEventClassWindow) {
		if (eventKind == kEventWindowClose) {
			HideWindow(dialog);
			EnableMenuCommand(NULL,'pref');
			result = noErr;
		}
	}
    else if(eventClass == kEventClassControl) {
		if (eventKind == kEventControlHit) {
			cID.signature = 'BOTR';
			cID.id = kStopItersStatText;
			GetControlByID(dialog, &cID, &stopItersStatCRef);
		    GetEventParameter (event, kEventParamDirectObject, typeControlRef, NULL, 
		                            sizeof (ControlRef), NULL, &controlRef);
		    GetControlID(controlRef, &controlID);
		   
		    switch (controlID.id)
		    {
		    	case kSetStopPtBox:
					boolVal = GetControlValue(controlRef);
					if (boolVal == true) {
						ActivateControl(stopItersStatCRef);
					}
					else {
						DeactivateControl(stopItersStatCRef);
					}
					break;
				case kSetDefaultButton:
					SetPrefsDialogVals(dialog, true);  // set dialog's values to deflt vals
					break;
				case kRevertButton:
					SetPrefsDialogVals(dialog, false);  // set dialog's values to prev vals
					break;
				case kHelpButton:
					HandleHelpPrefs();
					break;
		    }
		    
		    UpdateControls(dialog, NULL);
		    
		    if (controlID.id == kOKButton || controlID.id == kApplyButton) {
		    	if ((intval = getIntVal(dialog, kMaxItersText)) >= 0)
					maxIters = intval;
				cID.id = kSetStopPtBox;
				GetControlByID(dialog, &cID, &controlRef);
				setStopPt = GetControlValue(controlRef);
				if ((intval = getIntVal(dialog, kStopItersText)) >= 0)
					stopIters = intval;
					if ((dblval = getDoubleVal(dialog, kSlowText)) >= MINTIMERDELAY &&
						dblval <= MAXTIMERDELAY)
					Slow = dblval;
				if ((dblval = getDoubleVal(dialog, kFastText)) >= MINTIMERDELAY &&
						dblval <= MAXTIMERDELAY)
					Fast = dblval;
				if ((dblval = getDoubleVal(dialog, kTurboText)) >= MINTIMERDELAY &&
						dblval <= MAXTIMERDELAY)
					Turbo = dblval;
				if (speed == iSlow)
					delay = Slow;
				else if (speed == iFast)
					delay = Fast;
				else if (speed == iTurbo)
					delay = Turbo;
				if (stop == 0) {
					RemoveEventLoopTimer(theTimer);
					InstallEventLoopTimer(GetMainEventLoop(), 0, delay, myTimer, NULL, &theTimer);
				}
				if ((intval = getIntVal(dialog, kINumOrgsText)) >= 0 &&
													intval <= WLSIZE * WLSIZE)
					iNumOrgs = intval;
				cID.id = kRandINumOrgsBox;
				GetControlByID(dialog, &cID, &controlRef);
				randINumOrgs = GetControlValue(controlRef);
				cID.id = kRerunSavedSimBox;
				GetControlByID(dialog, &cID, &controlRef);
				rerunSavedSim = GetControlValue(controlRef);
				cID.id = kRandomizeProgBox;
				GetControlByID(dialog, &cID, &controlRef);
				randomizeProg = GetControlValue(controlRef);
						
				savePrefs();
				/*if (maxOrgs != oldmaxorgs)
					orgs = realloc(orgs, 2 * maxOrgs * sizeof(org));*/
		    }
		    if (controlID.id == kOKButton) {
				HideWindow(dialog);
				EnableMenuCommand(NULL,'pref');
			}
		    
		}
		result = noErr;
	}
	else if (eventClass == kEventClassKeyboard && (eventKind == kEventRawKeyDown
					|| eventKind == kEventRawKeyRepeat)) {
		ConvertEventRefToEventRecord(event, &evtRec);
		charCode = evtRec.message & charCodeMask;
		if (charCode == kEscapeCharCode) {
			HideWindow(dialog);
			EnableMenuCommand(NULL,'pref');
		}
		//if ((charCode == kReturn) || (charCode == kEnter))
				//return noErr;
		HandleMenuChoice(MenuEvent(&evtRec));
	}
	
    return result;
}

int getIntVal(WindowRef dialog, int textItem)
{
	int		ival = -1;
	char	tmpcstr[16];
	Size	sz;
	ControlRef	cRef;
	ControlID	cID;
	
	cID.signature = 'BOTR';
	cID.id = textItem;
	GetControlByID(dialog, &cID, &cRef);
	GetControlData(cRef, kControlEntireControl, kControlEditTextTextTag,
						sizeof(tmpcstr), &tmpcstr, &sz);
	tmpcstr[sz] = '\0';
	sscanf(tmpcstr, " %d", &ival);  // if unsucessful, remains -1
	return ival;
}

double getDoubleVal(WindowRef dialog, int textItem)
{
	double	dval = -1.0;
	char	tmpcstr[16];
	Size	sz;
	ControlRef	cRef;
	ControlID	cID;
	
	cID.signature = 'BOTR';
	cID.id = textItem;
	GetControlByID(dialog, &cID, &cRef);
	GetControlData(cRef, kControlEntireControl, kControlEditTextTextTag,
						sizeof(tmpcstr), &tmpcstr, &sz);
	tmpcstr[sz] = '\0';
	sscanf(tmpcstr, " %lf", &dval);  // if unsucessful, remains -1
	return dval;
}

void SetPrefsDialogVals(WindowRef dialog, Boolean defaultVal)
{
	CFStringRef  text;
	ControlRef	controlRef, stopItersStatCRef;//, stopGensStatCRef, stopOrgsStatCRef;
	ControlEditTextSelectionRec selRec;
	ControlRef	cRef;
	ControlID	cID;
	Size txtSize, slSize;
	short	prevStart, prevEnd;
	Boolean wasEntirelySelected = false;
	
	cID.signature =  kAppSignature;
	selRec.selStart = 0;
	selRec.selEnd = 0; 
	GetKeyboardFocus(dialog, &cRef);
	GetControlDataSize(cRef, kControlEditTextPart, kControlEditTextTextTag, &txtSize);
	GetControlData(cRef, kControlEditTextPart, kControlEditTextSelectionTag,
					sizeof(ControlEditTextSelectionRec), &selRec, &slSize );
	prevStart = selRec.selStart;
	prevEnd = selRec.selEnd;
	if (txtSize <= selRec.selEnd - selRec.selStart)
		wasEntirelySelected = true;
	if (defaultVal)
		text = CFStringCreateWithFormat( NULL, NULL, CFSTR("%d"), MAXITERS );
	else
		text = CFStringCreateWithFormat( NULL, NULL, CFSTR("%d"), maxIters );
	cID.id = kMaxItersText;
	GetControlByID(dialog, &cID, &cRef);
	SetControlData( cRef, 0, kControlEditTextCFStringTag, sizeof(CFStringRef), &text );
	cID.id = kSetStopPtBox;
	GetControlByID(dialog, &cID, &controlRef);
	if (defaultVal)
		SetControlValue(controlRef, SETSTOPPT);
	else
		SetControlValue(controlRef, setStopPt);
	cID.id = kStopItersStatText;
	GetControlByID(dialog, &cID, &stopItersStatCRef);
	if ((defaultVal && !SETSTOPPT) || (!defaultVal && !setStopPt)) {
		DeactivateControl(stopItersStatCRef);
	}
	else {
		ActivateControl(stopItersStatCRef);
	}
	if (defaultVal)
		text = CFStringCreateWithFormat( NULL, NULL, CFSTR("%d"), STOPITERS );
	else
		text = CFStringCreateWithFormat( NULL, NULL, CFSTR("%d"), stopIters );
	cID.id = kStopItersText;
	GetControlByID(dialog, &cID, &cRef);
	SetControlData( cRef, 0, kControlEditTextCFStringTag, sizeof(CFStringRef), &text );
	if (defaultVal)
		text = CFStringCreateWithFormat( NULL, NULL, CFSTR("%g"), SLOW );
	else
		text = CFStringCreateWithFormat( NULL, NULL, CFSTR("%g"), Slow );
	cID.id = kSlowText;
	GetControlByID(dialog, &cID, &cRef);
	SetControlData( cRef, 0, kControlEditTextCFStringTag, sizeof(CFStringRef), &text );
	if (defaultVal)
		text = CFStringCreateWithFormat( NULL, NULL, CFSTR("%g"), FAST );
	else
		text = CFStringCreateWithFormat( NULL, NULL, CFSTR("%g"), Fast );
	cID.id = kFastText;
	GetControlByID(dialog, &cID, &cRef);
	SetControlData( cRef, 0, kControlEditTextCFStringTag, sizeof(CFStringRef), &text );
	if (defaultVal)
		text = CFStringCreateWithFormat( NULL, NULL, CFSTR("%g"), TURBO );
	else
		text = CFStringCreateWithFormat( NULL, NULL, CFSTR("%g"), Turbo );
	cID.id = kTurboText;
	GetControlByID(dialog, &cID, &cRef);
	SetControlData( cRef, 0, kControlEditTextCFStringTag, sizeof(CFStringRef), &text );
	if (defaultVal)
		text = CFStringCreateWithFormat( NULL, NULL, CFSTR("%d"), INUMORGS );
	else
		text = CFStringCreateWithFormat( NULL, NULL, CFSTR("%d"), iNumOrgs );
	cID.id = kINumOrgsText;
	GetControlByID(dialog, &cID, &cRef);
	SetControlData( cRef, 0, kControlEditTextCFStringTag, sizeof(CFStringRef), &text );
	cID.id = kRandINumOrgsBox;
	GetControlByID(dialog, &cID, &controlRef);
	if (defaultVal)
		SetControlValue(controlRef, RANDINUMORGS);
	else
		SetControlValue(controlRef, randINumOrgs);
	cID.id = kRerunSavedSimBox;
	GetControlByID(dialog, &cID, &controlRef);
	if (defaultVal)
		SetControlValue(controlRef, RERUNSAVEDSIM);
	else
		SetControlValue(controlRef, rerunSavedSim);
	cID.id = kRandomizeProgBox;
	GetControlByID(dialog, &cID, &controlRef);
	if (defaultVal)
		SetControlValue(controlRef, RANDOMIZEPROG);
	else
		SetControlValue(controlRef, randomizeProg);
	SetKeyboardFocus(dialog, controlRef, kControlFocusNextPart);
	GetKeyboardFocus(dialog, &cRef);
	GetControlDataSize(cRef, kControlEditTextPart, kControlEditTextTextTag, &txtSize);
	GetControlData(cRef, kControlEditTextPart, kControlEditTextSelectionTag,
						sizeof(ControlEditTextSelectionRec), &selRec, &slSize );
	if (wasEntirelySelected) {
		selRec.selStart	= 0;
		selRec.selEnd = txtSize;  // if all was selcd -> shrunk -> revertd: sel all
	}
	else if (txtSize <= prevStart)
		selRec.selEnd = selRec.selStart = txtSize;
	else {
		selRec.selStart = prevStart;
		if (txtSize <= prevEnd)
			selRec.selEnd = txtSize;
		else
			selRec.selEnd = prevEnd;
	}
	SetControlData(cRef, kControlEntireControl, kControlEditTextSelectionTag,
						sizeof(ControlEditTextSelectionRec), &selRec);
}

void HandleHelpPrefs(void)
{
	EventTypeSpec 	windowSpec[] = { {kEventClassKeyboard, kEventRawKeyDown},
											  {kEventClassWindow, kEventWindowClose} };
	EventHandlerUPP	windowUPP;
	static WindowRef helpPrefsWindow = NULL;
	OSStatus      err=noErr;
	ControlID id = { 'HITV', 100 }; // set with Interface Builder in the Inspector
	HIViewRef textView;
	
	
	if (helpPrefsWindow == NULL) {
		// Create a window. "Help" is the name of the window object. This name is set in 
		// InterfaceBuilder when the nib is created.
		err = CreateWindowFromNib( gNibs, CFSTR("Help"), &helpPrefsWindow );
		
		HIViewFindByID(HIViewGetRoot(helpPrefsWindow), id, &textView);
		
		// let's get the URL to our .rtf file
		CFURLRef theURL = CFBundleCopyResourceURL(CFBundleGetMainBundle(), CFSTR("Help"), CFSTR("rtf"), NULL);
		TXNReadFromCFURL(HITextViewGetTXNObject(textView), kTXNStartOffset, kTXNStartOffset, NULL, theURL, NULL);
		
		CFRelease(theURL);
		
		windowUPP =  NewEventHandlerUPP(HelpPrefsWindowEventHandler);
		err = InstallWindowEventHandler(helpPrefsWindow, windowUPP, GetEventTypeCount(windowSpec),
												  windowSpec, (void *) helpPrefsWindow, NULL);
	}
	// The window was created hidden, so show it
	ShowWindow( helpPrefsWindow );
	SelectWindow(helpPrefsWindow);

}

OSStatus HelpPrefsWindowEventHandler(EventHandlerCallRef myHandler, EventRef event,
						void *userData)
{
	#pragma unused (myHandler)
    OSStatus 	result = eventNotHandledErr;
    UInt32		eventClass, eventKind;
    WindowRef	helpWindow	= userData;
    EventRecord	evtRec;  
    SInt8		charCode;
    
    eventClass = GetEventClass(event);
    eventKind = GetEventKind(event);
    
	if (eventClass == kEventClassWindow && eventKind == kEventWindowClose) {
		HideWindow(helpWindow);
		result = noErr;
	}
	else if (eventClass == kEventClassKeyboard && eventKind == kEventRawKeyDown) {
		ConvertEventRefToEventRecord(event, &evtRec);
		charCode = evtRec.message & charCodeMask;
		if (charCode == kEscapeCharCode) {
			HideWindow(helpWindow);
			result = noErr;
		}
	}	

    return result;
}

void HandleInContent(EventRecord *eventPtr, Point globalPoint, WindowRef theWindow)
{
	Point where, mouseLoc;
	MouseTrackingResult trackingResult;
	GrafPtr oldPort;
	Boolean portChanged = QDSwapPort(GetWindowPort(theWindow), &oldPort);
	int x, y, mlh, mlv, dir, i;
	Boolean mousePressedMoved = false, disappeared = false;
	Boolean deselect = false, clickedAnOrg = false;
	org tmp = {0, 0};
	int orgUnderneathOldIx, orgUnderneathNewIx;
	
	where = globalPoint;
	GlobalToLocal(&where);
		
	if (theWindow == window) {
		if ((where.h >= ACTVSTART) && (where.v >= ACTVSTART) &&
			(where.h <= ACTVEND) && (where.v <= ACTVEND)) {
			
			x = (where.h - ACTVSTART) / CELLSIZE;
			y = (where.v - ACTVSTART) / CELLSIZE;
			
			if (find_org(x, y) == -1)
				clickedAnOrg = false;
			else
				clickedAnOrg = true;
				
			// don't select/deselect/drag orgs while running in hyper
			// + do it only if we clicked on an org
			if (!(speed == iHyper && stop == 0) && clickedAnOrg) {
				
				if (prinfo == 0 || x != selcx || y != selcy) {
					// if we clicked on a non-selected org
					// i.e  if nothing was selected or another org was selected
					deselect = false;   // select organism here
					prinfo = 1;
					draw_frame(selcx, selcy, 0);   // remove old frame
					selcx = x;
					selcy = y;
					draw_frame(selcx, selcy, 1);
					selectOrgIndex = find_org(selcx, selcy);
					outext2();
					aglSwapBuffers(myAGLContext);
				}
				else {   // deselect "prototype"
					// if we chose an already selected org
					deselect = true;   // deselect organism
				}
					
				// dragging/cloning "prototype"
				// we don't want to initiate a drag if we clicked on a moving
				// org and started dragging after the org had already moved.
				// we do this by rechecking the world array, which may have
				// changed since we set the clickedAnOrg variable and hence
				// we don't use that var
				if (WaitMouseMoved(eventPtr->where) && clickedAnOrg)
					mousePressedMoved = true;
				else if (deselect == true) {   // deselect "definition"
					prinfo = 0;
					draw_frame(selcx, selcy, 0);   // remove old frame
					clr_selorg_text();
					selectOrgIndex = -1;
				}
			}
			
			// stop/continue simulation
			if (stop == 0 && (speed == iHyper || !clickedAnOrg)) {   // stop sim
				stop = 1;
				cont_enabled = 1;
				DisableMenuItem(Fmenu, iStop);
				EnableMenuItem(Fmenu, iContinue);
				if (speed == iHyper || limited_FPS == 1) {
					redraw_scr();
					outext();
				}
			}
			else if (cont_enabled == 1 && !clickedAnOrg) {   // continue sim
				if (speed == iStep && end_sim == 0) {
					HandleDraw();
					stop = 1;
				}
				else {
					stop = 0;
					cont_enabled = 0;
					DisableMenuItem(Fmenu, iContinue);
					EnableMenuItem(Fmenu, iStop);
				}
			}
			
			//   dragging/cloning "definition"
			if (mousePressedMoved) {
				dir = orgs[selectOrgIndex].direc;
				trackingResult = kMouseTrackingMouseDown;
				while (trackingResult != kMouseTrackingMouseUp) {
					TrackMouseLocation(NULL, &mouseLoc, &trackingResult);
					mlh = mouseLoc.h;
					mlv = mouseLoc.v;
					if ((mlh >= ACTVSTART) && (mlv >= ACTVSTART) &&
							(mlh <= ACTVEND) && (mlv <= ACTVEND)) {
						if (mlh <= ACTVSTART + CELLSIZE / 2)
							mlh = ACTVSTART + CELLSIZE / 2;
						else if (mlh >= ACTVEND - CELLSIZE / 2)
							mlh = ACTVEND - CELLSIZE / 2;
						if (mlv <= ACTVSTART + CELLSIZE / 2)
							mlv = ACTVSTART + CELLSIZE / 2;
						else if (mlv >= ACTVEND - CELLSIZE / 2)
							mlv = ACTVEND - CELLSIZE / 2;
						x = (mlh - ACTVSTART) / CELLSIZE;
						y = (mlv - ACTVSTART) / CELLSIZE;
						redraw_scr();
						if (selectOrgIndex == -1)  // if the org died while we were dragging it
							break;
						draw_frame(selcx, selcy, 1);  // erase frame
						DrawOrgPrecisely(dir, mlh, mlv);
						outext3(x, y, 2);  // in blue
					}
					else {
						disappeared = true;
						break;
					}
				}
				clr_dragged_org_text();
				if (selectOrgIndex != -1) {   // if the org didn't die while we were dragging it
					// x,y = new location (where we will drop selorg)
					// selcx,selcy = old location (where we started the drag)
					if (disappeared == false && (x != selcx || y != selcy)) { // if moved & !offbounds
						tmp = orgs[selectOrgIndex];
						// find org in top layer at (x,y) where we want to drop selorg
						// (this is the last org in the orgs array which has x,y coordinates)
						orgUnderneathNewIx = find_org(x, y);
						if (orgUnderneathNewIx != -1) {  // if there's an org in the new location
							// ouni is highest layer org in (x,y) where selorg will be.
							// insert tmp (== selorg) right after ouni, so that it'll be
							// drawn right on top of ouni. (since we draw orgs from beginning
							// of array in first layers and last org in the array on top of
							// existing orgs).
							
							if (orgUnderneathNewIx > selectOrgIndex) {
								for (i = selectOrgIndex; i < orgUnderneathNewIx; i++)
									orgs[i] = orgs[i + 1];
								orgs[orgUnderneathNewIx] = tmp;
								selectOrgIndex = orgUnderneathNewIx;
								orgUnderneathNewIx--;
							}
							// if (ouni == selorgix) is impossible
							// if (ouni < selorgix) then soi is already in the right place
						}
						orgs[selectOrgIndex].x = x;
						orgs[selectOrgIndex].y = y;
					
					clr_selorg_text();
						outext2();
						selcx = x;  // make the current coordinates of selorg = drop coords
						selcy = y;
					}
					redraw_scr();
					draw_frame(orgs[selectOrgIndex].x, orgs[selectOrgIndex].y, 1);
				}
			}
						
		}
		
		aglSwapBuffers(myAGLContext);
	}
	if (portChanged)
		QDSwapPort(oldPort, NULL);
}

void DrawOrgPrecisely(int dir, int x, int y)
{  // for drawing a sprite when we drag it (pixel accuracy)
	long int	newLeft, newTop;

	newLeft = x - CELLSIZE / 2;
	newTop = y - CELLSIZE / 2;
	
	glBindTexture(GL_TEXTURE_RECTANGLE_EXT, texture[dir]);
	//glTranslatef(20,20,0);
	glBegin(GL_QUADS); // try clockwise and anti c-w to see which is faster
	
	glTexCoord2f(0, 0);
	glVertex2f(newLeft, newTop);
	
	glTexCoord2f(CELLSIZE, 0);
	glVertex2f(newLeft + CELLSIZE, newTop);
	
	glTexCoord2f(CELLSIZE, CELLSIZE);
	glVertex2f(newLeft + CELLSIZE, newTop + CELLSIZE);
	
	glTexCoord2f(0, CELLSIZE);
	glVertex2f(newLeft, newTop + CELLSIZE);
	
	glEnd();
	aglSwapBuffers(myAGLContext);
}
	
void HandleDraw(void)
{	
	// this is where all the action is.
	int orgCnt;//, oldnorgs;
	//int prinfo_term;  // should we print to the terminal
	int i, j, posx, posy, dx, dy, dirc;//, imax, jmax, sum;
	///double propor;
	GrafPtr oldPort;
	Boolean portChanged = QDSwapPort(GetWindowPort(window), &oldPort);
	
	
	iters++;
	if (limit_to_60FPS && delay < 0.016667)
		limited_FPS = 1;
	else {
		if (limited_FPS == 1)
			redraw_scr();
		limited_FPS = 0;
	}
	if (prinfo == 1) {
		draw_frame(selcx, selcy, 0);
		if (limited_FPS == 1) {
			prinfo = 0;
			clr_selorg_text();
			selectOrgIndex = -1;
		}
	}
	/* update org info */
	//oldnorgs = norgs;
	orgCnt = 0;
	while (orgCnt < norgs) {
		// clear all orgs from world. if we dragged org on food - leave food.
		// only if org advanced by itself (via handledraw) on food - eat it &
		// erase it from new position (see below).
			if (speed != iHyper && limited_FPS == 0) {
				DrawGreen(orgs[orgCnt].x, orgs[orgCnt].y);
				if (world[orgs[orgCnt].x][orgs[orgCnt].y] == FOOD)
					DrawIcon(FOOD, orgs[orgCnt].x, orgs[orgCnt].y);
			}
		dirc = orgs[orgCnt].direc;
		switch (dirc) {
		case UP:
			dx = 0;
			dy = -1;
			break;
		case DOWN:
			dx = 0;
			dy = 1;
			break;
		case LEFT:
			dx = -1;
			dy = 0;
			break;
		case RIGHT:
			dx = 1;
			dy = 0;
			break;
		default:
			//printf("error in direction\n");
			exit(1);
			break;
		}
		posx = orgs[orgCnt].x + dx;
		if (posx == WLSIZE)
			posx = 0;
		else if (posx < 0)
			posx = WLSIZE - 1;
		orgs[orgCnt].x = posx;
		posy = orgs[orgCnt].y + dy;
		if (posy == WLSIZE)
			posy = 0;
		else if (posy < 0)
			posy = WLSIZE - 1;
		orgs[orgCnt].y = posy;
		if (myrand() % MUTRATE == 0) {
			dirc = (dirc + 1) % 4;  // mutate direction
			orgs[orgCnt].direc = dirc;
		}
		
		orgCnt++;
	}  // end while
	if (speed != iHyper && limited_FPS == 0) {
		for (orgCnt = 0; orgCnt < norgs; orgCnt++) {
			//colorDiff = findColorDiff(orgCnt);
			posx = orgs[orgCnt].x;
			posy = orgs[orgCnt].y;
			DrawGreen(posx, posy);
			DrawIcon(orgs[orgCnt].direc, posx, posy);
		}
	}
	if (speed == iHyper && iters % 10000 == 0) 
		outext(); // we need this since hyper doesn't pass through the usual loop
	if (speed != iHyper) {
		if (prinfo == 1) {
			outext2();
			selcx = orgs[selectOrgIndex].x;
			selcy = orgs[selectOrgIndex].y;
			draw_frame(selcx, selcy, 1);
		}
		if (limited_FPS == 0) {
			if (delay > 0.001 || iters % 100 == 0) {
				outext(); // draw the text (includes aglSwapBufs)
			}
			//else if (prinfo_term == 1) // with this condition: draws every 100 frames except if events: every one
				aglSwapBuffers(myAGLContext); // draw the graphics
		}
		else if (timed_out_60FPS()) {
			redraw_scr();
			outext();
			timed_out_60FPS(); // reset timer to 0 secs, then continue iterating for 0.0167 secs
		}  // before entering the surrounding "else if" above to redraw picture
	}
	//total += norgs;
	if (iters >= maxIters) {
		stop = 1;
		cont_enabled = 0;
		end_sim = 1;
		DisableMenuItem(Fmenu, iStop);
		DisableMenuItem(Fmenu, iContinue);
		if (speed == iHyper || limited_FPS == 1) {
			redraw_scr();
			outext();
		}
	}
	else if (setStopPt && (iters == stopIters)) {
		stop = 1;
		cont_enabled = 1;
		DisableMenuItem(Fmenu, iStop);
		EnableMenuItem(Fmenu, iContinue);
		if (speed == iHyper || limited_FPS == 1) {
			redraw_scr();
			outext();
		}
	}
	if (portChanged)
		QDSwapPort(oldPort, NULL);
} 
				
void DrawIcon(int icn, int x, int y)
{
	long int	newLeft, newTop;

	newLeft = ACTVSTART + CELLSIZE * x;
	newTop = ACTVSTART + CELLSIZE * y;
	
	glBindTexture(GL_TEXTURE_RECTANGLE_EXT, texture[icn]);
	glBegin(GL_QUADS);
	
	glTexCoord2f(0, 0);
	glVertex2f(newLeft, newTop);
	
	glTexCoord2f(CELLSIZE, 0);
	glVertex2f(newLeft + CELLSIZE, newTop);
	
	glTexCoord2f(CELLSIZE, CELLSIZE);
	glVertex2f(newLeft + CELLSIZE, newTop + CELLSIZE);
	
	glTexCoord2f(0, CELLSIZE);
	glVertex2f(newLeft, newTop + CELLSIZE);
	
	glEnd();
}

void DrawGreen(int x, int y)
{
	long int	newLeft, newTop;

	newLeft = ACTVSTART + CELLSIZE * x;
	newTop = ACTVSTART + CELLSIZE * y;
	
	//glColor3ub(0, 128, 17);  // draw green square - color is already set to green
	glRecti(newLeft, newTop, newLeft + CELLSIZE, newTop + CELLSIZE);
}

void MoveOrgByArrowKeys(SInt8 charCode)
{
	int posx, posy, dx, dy, dir, i;
	GrafPtr oldPort;
	Boolean portChanged;
	org tmp = {0, 0};
	int orgUnderneathOldIx, orgUnderneathNewIx;
	
	switch (charCode) {
		case kUpArrowCharCode:
			dx = 0;
			dy = -1;
			break;
		case kDownArrowCharCode:
			dx = 0;
			dy = 1;
			break;
		case kLeftArrowCharCode:
			dx = -1;
			dy = 0;
			break;
		case kRightArrowCharCode:
			dx = 1;
			dy = 0;
			break;
		default:
			//printf("error in direction\n");
			exit(1);
			break;
	}
	dir = orgs[selectOrgIndex].direc;
	selcx = orgs[selectOrgIndex].x;
	posx = selcx + dx;
	if (posx == WLSIZE)
		posx = 0;
	else if (posx < 0)
		posx = WLSIZE - 1;
	selcy = orgs[selectOrgIndex].y;
	posy = selcy + dy;
	if (posy == WLSIZE)
		posy = 0;
	else if (posy < 0)
		posy = WLSIZE - 1;
	tmp = orgs[selectOrgIndex];
	// find org in top layer at (posx,posy) where we want to move selorg
	// (this is the last org in the orgs array which has posx,posy coordinates)
	orgUnderneathNewIx = find_org(posx, posy);
	if (orgUnderneathNewIx != -1) {  // if there's an org in the new location
		// ouni is highest layer org in (posx,posy) where selorg will be.
		// insert tmp (== selorg) right after ouni, so that it'll be
		// drawn right on top of ouni. (since we draw orgs from beginning
		// of array in first layers and last org in the array on top of
		// existing orgs).
		if (orgUnderneathNewIx < selectOrgIndex - 1) {
			for (i = selectOrgIndex; i >= orgUnderneathNewIx + 2; i--)
				orgs[i] = orgs[i - 1];
			orgs[orgUnderneathNewIx + 1] = tmp;
			selectOrgIndex = orgUnderneathNewIx + 1;
		}
		else if (orgUnderneathNewIx > selectOrgIndex) {
			for (i = selectOrgIndex; i < orgUnderneathNewIx; i++)
				orgs[i] = orgs[i + 1];
			orgs[orgUnderneathNewIx] = tmp;
			selectOrgIndex = orgUnderneathNewIx;
			orgUnderneathNewIx--;
		}
		// if (ouni == selorgix) is impossible
		// if (ouni == selorgix - 1) then soi is already in the right place
	}
	orgs[selectOrgIndex].x = posx;
	orgs[selectOrgIndex].y = posy;
	// find org undernth where selectOrg was.
	orgUnderneathOldIx = find_org(selcx, selcy);
	// if we want to drop selorg on ground or food
	selcx = posx;
	selcy = posy;
	portChanged = QDSwapPort(GetWindowPort(window), &oldPort);
	redraw_scr();
	draw_frame(posx, posy, 1);
	outext2();
	aglSwapBuffers(myAGLContext);
	if (portChanged)
		QDSwapPort(oldPort, NULL);
}

void draw_frame(int x, int y, int col)
{
	long int	newLeft, newTop;
	
	if (col == 1)
		glColor3ub(0, 0, 0); // black color
	else if (col == 0)
		glColor3ub(0, 128, 17);  // erase frame with green background color
	else if (col == 2)
		glColor3ub(0, 0, 255);  // blue
	
	newLeft = ACTVSTART + CELLSIZE * x;
	newTop = ACTVSTART + CELLSIZE * y;
	
	glDisable(GL_TEXTURE_RECTANGLE_EXT);
	glEnable(GL_TEXTURE_1D);
	glBegin(GL_LINE_STRIP);
	glVertex2f(newLeft, newTop + 1);
	glVertex2f(newLeft + CELLSIZE, newTop + 1);
	glVertex2f(newLeft + CELLSIZE, newTop + CELLSIZE);
	glVertex2f(newLeft, newTop + CELLSIZE);
	glVertex2f(newLeft, newTop + 1);
	glEnd();
	glDisable(GL_TEXTURE_1D);
	glEnable(GL_TEXTURE_RECTANGLE_EXT);
}

// C string drawing function
void drawCStringGL (char * cstrOut)
{
	GLint i = 0;
	while (cstrOut [i])
		glCallList (fontList + cstrOut[i++]);
}

void outext(void)
/* print text in main graphics wind. */
{
	char cstr [256];
	GLint line = 9;
	
	glDisable(GL_TEXTURE_RECTANGLE_EXT);
	glColor3f(1.0, 1.0, 1.0);
	glRecti(322, (line - 1) * 12, 436, (line + 2) * 12 + 4);
	// output strings
	glColor3f (0.0, 0.0, 0.0);
	glRasterPos3d (330, line++ * 12, 0);
	if (speed != iHyper && limited_FPS && delay < 0.001)
		sprintf(cstr, "iter = %d", 100 * (iters / 100));
	else
		sprintf(cstr, "iter = %d", iters);
	drawCStringGL (cstr);
	glEnable(GL_TEXTURE_RECTANGLE_EXT);
	aglSwapBuffers(myAGLContext);
}
	
			
void outext2(void)
/* print info on selected org in main window */
{
	char cstr [256];
	GLint line = 15;
	
	glDisable(GL_TEXTURE_RECTANGLE_EXT);
	glColor3f(1.0, 1.0, 1.0);
	glRecti(322, (line - 1) * 12, 420, (line + 6) * 12 + 2);
	// output strings
	glColor3f (0.0, 0.0, 0.0);
	glRasterPos3d (330, line++ * 12, 0);
	sprintf(cstr, "selected org:");
	drawCStringGL (cstr);
	line++;
	glRasterPos3d (330, line++ * 12, 0);
	sprintf(cstr, "x = %d", orgs[selectOrgIndex].x);
	drawCStringGL (cstr);
	glRasterPos3d (330, line++ * 12, 0);
	sprintf(cstr, "y = %d", orgs[selectOrgIndex].y);
	drawCStringGL (cstr);
	glEnable(GL_TEXTURE_RECTANGLE_EXT);
	aglSwapBuffers(myAGLContext);
}

void outext3(int x, int y, int col)
/* print info on dragged org in main window */
{
	char cstr [256];
	GLint line = 22;
	
	glDisable(GL_TEXTURE_RECTANGLE_EXT);
	glColor3f(1.0, 1.0, 1.0);
	glRecti(322, (line - 1) * 12 + 2, 420, (line + 1) * 12 + 4);
	if (col == 2)
		glColor3f(0.0, 0.0, 1.0); // blue color
	glRasterPos3d (330, line++ * 12, 0);
	sprintf(cstr, "x = %d", x);
	drawCStringGL (cstr);
	glRasterPos3d (330, line++ * 12, 0);
	sprintf(cstr, "y = %d", y);
	drawCStringGL (cstr);
	glEnable(GL_TEXTURE_RECTANGLE_EXT);
	aglSwapBuffers(myAGLContext);
}

void outext4(double fps)
/* print FPS or IPS (iters per sec) in main window */
{
	GrafPtr oldPort;
	Boolean portChanged = QDSwapPort(GetWindowPort(window), &oldPort);
	char cstr [256];
	GLint line = 4;
	
	glDisable(GL_TEXTURE_RECTANGLE_EXT);
	glColor3f(1.0, 1.0, 1.0);
	glRecti(322, (line - 1) * 12, 430, line * 12 + 2);
	glColor3f(0.0, 0.0, 0.0);
	glRasterPos3d (330, line++ * 12, 0);
	if (speed == iHyper || (limited_FPS == 1 && speed != iStep)) {
		if (fps < 100.0)
			sprintf(cstr, "%.2f CPS", fps);
		else
			sprintf(cstr, "%.0f CPS", fps);
	}
	else {
		if (fps < 100.0)
			sprintf(cstr, "%.2f FPS", fps);
		else
			sprintf(cstr, "%.0f FPS", fps);
	}
	drawCStringGL (cstr);
	glEnable(GL_TEXTURE_RECTANGLE_EXT);
	aglSwapBuffers(myAGLContext);
	if (portChanged)
		QDSwapPort(oldPort, NULL);
}

void clr_selorg_text(void)
/* clears outext2 when needed */
{
	GrafPtr oldPort;
	Boolean portChanged = QDSwapPort(GetWindowPort(window), &oldPort);
	GLint line = 15;
	
	glDisable(GL_TEXTURE_RECTANGLE_EXT);
	glColor3f(1.0, 1.0, 1.0);
	glRecti(322, (line - 1) * 12, 420, (line + 6) * 12 + 2);
	glEnable(GL_TEXTURE_RECTANGLE_EXT);
	aglSwapBuffers(myAGLContext);
	if (portChanged)
		QDSwapPort(oldPort, NULL);
}

void clr_dragged_org_text(void)
/* clears outext3 when needed */
{
	// the calling function already sets the port 
	GLint line = 22;
	
	glDisable(GL_TEXTURE_RECTANGLE_EXT);
	glColor3f(1.0, 1.0, 1.0);
	glRecti(322, (line - 1) * 12, 420, (line + 6) * 12 + 2);
	glEnable(GL_TEXTURE_RECTANGLE_EXT);
	aglSwapBuffers(myAGLContext);
}

void clr_FPS_text(void)
/* clears outext4 when needed */
{
	GrafPtr oldPort;
	Boolean portChanged = QDSwapPort(GetWindowPort(window), &oldPort);
	
	GLint line = 4;
	
	glDisable(GL_TEXTURE_RECTANGLE_EXT);
	glColor3f(1.0, 1.0, 1.0);
	glRecti(322, (line - 1) * 12, 430, line * 12 + 2);
	glEnable(GL_TEXTURE_RECTANGLE_EXT);
	aglSwapBuffers(myAGLContext);
	if (portChanged)
		QDSwapPort(oldPort, NULL);
}

void calcFPS(void)
{
	static AbsoluteTime sLastFlush = {0, 0};
	AbsoluteTime curTime;
	Nanoseconds delta;
	double d = 0;

	curTime = UpTime();
	delta = AbsoluteDeltaToNanoseconds(curTime, sLastFlush);
	d = (UnsignedWideToUInt64(delta)) / (double)1E9;
	if (d >= 1.0) {
		outext4((iters - last_iters) / d);
		sLastFlush = curTime;
		last_iters = iters;
	}
}

int timed_out_60FPS(void)
{
	static AbsoluteTime sLastFlush = {0, 0};
	AbsoluteTime curTime;
	Nanoseconds delta;
	double d = 0;
	int timed_out = 0;
	static int wait_redraw = 0;

	curTime = UpTime();
	delta = AbsoluteDeltaToNanoseconds(curTime, sLastFlush);
	d = (UnsignedWideToUInt64(delta)) / (double)1E9;
	if (d >= 0.016667 || wait_redraw == 1) {
		timed_out = 1;
		sLastFlush = curTime;
		wait_redraw = !wait_redraw;  // now wait for redraw, then reset timer
		// we want to time 33 msecs between redraws, not including redraws
	}
	return timed_out;
}

void redraw_scr(void)
{
/* redraw entire screen */
	GrafPtr oldPort;
	Boolean portChanged = QDSwapPort(GetWindowPort(window), &oldPort);
	int i, j;
        
	//glColor3ub(0, 128, 17);  // draw green square
	glRecti(10, 10, 290, 290);
	for (i = 0; i < WLSIZE; i++) {  // draw ground and food world
		for (j = 0; j < WLSIZE; j++) {
			if (world[i][j] == GROUND)
				DrawGreen(i, j);
			else
				DrawIcon(world[i][j], i, j);
		}
	}
	for (i = 0; i < norgs; i++) // add orgs layer/s
		DrawIcon(orgs[i].direc, orgs[i].x, orgs[i].y);
	aglSwapBuffers(myAGLContext);
	if (portChanged)
		QDSwapPort(oldPort, NULL);
}

/*void create_world(void)
// allocate memory for world 
{
	int i, j;
	
	// make space for 2-D world array 
   	if ((world = (int **) calloc(WLSIZE, sizeof(int *))) == NULL) {
     	 printf("Error: calloc failed for world array\n");
      	exit(1);
   	}
   	for (i = 0; i < WLSIZE; i++)
      	if ((world[i] = (int *) calloc(WLSIZE, sizeof(int))) == NULL) {
         	printf("Error: calloc failed for world[i]\n");
         	exit(1);
      	}
}*/

void getPrefs(void)
{
	CFStringRef	slowKey	= CFSTR("slow");
	CFStringRef	fastKey	= CFSTR("fast");
	CFStringRef	turboKey	= CFSTR("turbo");
	CFStringRef	variableKey	= CFSTR("variable");
	CFStringRef	sliderValKey	= CFSTR("sliderVal");
	CFStringRef	maxItersKey	= CFSTR("maxIters");
	CFStringRef	setStopPtKey	= CFSTR("setStopPt");
	CFStringRef	stopItersKey	= CFSTR("stopIters");
	CFStringRef	iNumOrgsKey	= CFSTR("iNumOrgs");
	CFStringRef	randINumOrgsKey	= CFSTR("randINumOrgs");
	CFStringRef	rerunSavedSimKey	= CFSTR("rerunSavedSim");
	CFStringRef	savedSeedKey	= CFSTR("savedSeed");
	CFStringRef	randomizeProgKey	= CFSTR("randomizeProg");
	CFNumberRef dblRef;
	Boolean success;
	
	maxIters = CFPreferencesGetAppIntegerValue(maxItersKey, kAppCFStr, &success);
	if (!success)
		maxIters = MAXITERS;
	setStopPt = CFPreferencesGetAppBooleanValue(setStopPtKey, kAppCFStr, &success);
	if (!success)
		setStopPt = SETSTOPPT;
	stopIters = CFPreferencesGetAppIntegerValue(stopItersKey, kAppCFStr, &success);
	if (!success)
		stopIters = STOPITERS;
		
	dblRef = CFPreferencesCopyAppValue(slowKey, kAppCFStr);
	if (dblRef != NULL)
		success = CFNumberGetValue(dblRef, kCFNumberDoubleType, &Slow);
	else
		Slow = SLOW;
	dblRef = CFPreferencesCopyAppValue(fastKey, kAppCFStr);
	if (dblRef != NULL)
		success = CFNumberGetValue(dblRef, kCFNumberDoubleType, &Fast);
	else
		Fast = FAST;
	dblRef = CFPreferencesCopyAppValue(turboKey, kAppCFStr);
	if (dblRef != NULL)
		success = CFNumberGetValue(dblRef, kCFNumberDoubleType, &Turbo);
	else
		Turbo = TURBO;
	dblRef = CFPreferencesCopyAppValue(variableKey, kAppCFStr);
	if (dblRef != NULL)
		success = CFNumberGetValue(dblRef, kCFNumberDoubleType, &Variable);
	else
		Variable = VARIABLE;
	sliderVal = CFPreferencesGetAppIntegerValue(sliderValKey, kAppCFStr, &success);
	if (!success)
		sliderVal = SLIDERVAL;
	iNumOrgs = CFPreferencesGetAppIntegerValue(iNumOrgsKey, kAppCFStr, &success);
	if (!success)
		iNumOrgs = INUMORGS;
	randINumOrgs = CFPreferencesGetAppBooleanValue(randINumOrgsKey, kAppCFStr, &success);
	if (!success)
		randINumOrgs = RANDINUMORGS;
	rerunSavedSim = CFPreferencesGetAppBooleanValue(rerunSavedSimKey, kAppCFStr, &success);
	if (!success)
		rerunSavedSim = RERUNSAVEDSIM;
	savedSeed = CFPreferencesGetAppIntegerValue(savedSeedKey, kAppCFStr, &success);
	if (!success)
		savedSeed = SAVEDSEED;
	randomizeProg = CFPreferencesGetAppBooleanValue(randomizeProgKey, kAppCFStr, &success);
	if (!success)
		randomizeProg = RANDOMIZEPROG;
	delay = Slow;
	if(dblRef != NULL)
		CFRelease(dblRef);
}

void savePrefs(void)
{
	CFStringRef	slowKey	= CFSTR("slow");
	CFStringRef	fastKey	= CFSTR("fast");
	CFStringRef	turboKey	= CFSTR("turbo");
	CFStringRef	variableKey	= CFSTR("variable");
	CFStringRef	sliderValKey	= CFSTR("sliderVal");
	CFStringRef	maxItersKey	= CFSTR("maxIters");
	CFStringRef	setStopPtKey	= CFSTR("setStopPt");
	CFStringRef	stopItersKey	= CFSTR("stopIters");
	CFStringRef	iNumOrgsKey	= CFSTR("iNumOrgs");
	CFStringRef	randINumOrgsKey	= CFSTR("randINumOrgs");
	CFStringRef	rerunSavedSimKey	= CFSTR("rerunSavedSim");
	CFStringRef	randomizeProgKey	= CFSTR("randomizeProg");
	CFStringRef	yes	= CFSTR("yes");
	CFStringRef	no	= CFSTR("no");
	CFNumberRef numRef, dblRef;
	
	numRef = CFNumberCreate(NULL, kCFNumberSInt32Type, &maxIters);
	CFPreferencesSetAppValue(maxItersKey, numRef, kAppCFStr);
	if(setStopPt == true)
		CFPreferencesSetAppValue(setStopPtKey, yes, kAppCFStr);
	else
		CFPreferencesSetAppValue(setStopPtKey, no, kAppCFStr);
	numRef = CFNumberCreate(NULL, kCFNumberSInt32Type, &stopIters);
	CFPreferencesSetAppValue(stopItersKey, numRef, kAppCFStr);
	dblRef = CFNumberCreate(NULL, kCFNumberDoubleType, &Slow);
	CFPreferencesSetAppValue(slowKey, dblRef, kAppCFStr);
	dblRef = CFNumberCreate(NULL, kCFNumberDoubleType, &Fast);
	CFPreferencesSetAppValue(fastKey, dblRef, kAppCFStr);
	dblRef = CFNumberCreate(NULL, kCFNumberDoubleType, &Turbo);
	CFPreferencesSetAppValue(turboKey, dblRef, kAppCFStr);
	dblRef = CFNumberCreate(NULL, kCFNumberDoubleType, &Variable);
	CFPreferencesSetAppValue(variableKey, dblRef, kAppCFStr);
	numRef = CFNumberCreate(NULL, kCFNumberSInt16Type, &sliderVal);
	CFPreferencesSetAppValue(sliderValKey, numRef, kAppCFStr);
	numRef = CFNumberCreate(NULL, kCFNumberSInt32Type, &iNumOrgs);
	CFPreferencesSetAppValue(iNumOrgsKey, numRef, kAppCFStr);
	if(randINumOrgs == true)
		CFPreferencesSetAppValue(randINumOrgsKey, yes, kAppCFStr);
	else
		CFPreferencesSetAppValue(randINumOrgsKey, no, kAppCFStr);
	if(rerunSavedSim == true)
		CFPreferencesSetAppValue(rerunSavedSimKey, yes, kAppCFStr);
	else
		CFPreferencesSetAppValue(rerunSavedSimKey, no, kAppCFStr);
	saveRandSeed();
	if(randomizeProg == true)
		CFPreferencesSetAppValue(randomizeProgKey, yes, kAppCFStr);
	else
		CFPreferencesSetAppValue(randomizeProgKey, no, kAppCFStr);
		
	CFPreferencesAppSynchronize(kAppCFStr);
	
	if (numRef != NULL)
		CFRelease(numRef);
	if(dblRef != NULL)
		CFRelease(dblRef);
}

void saveRandSeed(void)
{
	CFStringRef	savedSeedKey	= CFSTR("savedSeed");
	CFNumberRef numRef;
	
	numRef = CFNumberCreate(NULL, kCFNumberSInt32Type, &savedSeed);
	CFPreferencesSetAppValue(savedSeedKey, numRef, kAppCFStr);
	CFPreferencesAppSynchronize(kAppCFStr);

	if (numRef != NULL)
		CFRelease(numRef);
}

void init_world(void)
{
	int i, j;
	
	for (i = 0; i < WLSIZE; i++) {
		for (j = 0; j < WLSIZE; j++) {
			world[i][j] = GROUND;
		}
	} 
	if (randINumOrgs)
		norgs = myrand() % iNumOrgs + 1;
	else
		norgs = iNumOrgs;
	for (i = 0; i < norgs; i++) {
		rand_dat_org(i);
	}
	/* initialize food */
	/* put food in about a quarter of the cells which */
	/* do not contain orgs */
	for (i = 0; i < WLSIZE; i++)
		for (j = 0; j < WLSIZE; j++) {
			if (world[i][j] == GROUND && myrand() % INITFOOD == 0) {
				world[i][j] = FOOD;
			}
		}
}

void new_sim(void)
{	

	iters = 0;
	last_iters = 0;
	prev_step_stop = 1;
	if (rerunSavedSim)
		seed = savedSeed;
	lastSeed = seed;
	init_world();
	total = norgs;
	SetPortWindowPort(window);
	prinfo = 0;
	selectOrgIndex = -1;
	if (maxIters == 0) {
		stop = 1;
		end_sim = 1;
		DisableMenuItem(Fmenu, iStop);
		DisableMenuItem(Fmenu, iContinue);
	}
	else {
		if (setStopPt && (stopIters == 0) ||
				speed == iStep) {
			stop = 1;
			cont_enabled = 1;
			DisableMenuItem(Fmenu, iStop);
			EnableMenuItem(Fmenu, iContinue);
		}
		else { // start new sim running, including after stopping at stopiters != 0
			stop = 0;  
			cont_enabled = 0;
			EnableMenuItem(Fmenu, iStop);
			DisableMenuItem(Fmenu, iContinue);
		}
		end_sim = 0;
	}
	simNum++;
	/* if you want next sim to start frozen then you can either */
	/* select step menu before new, or set a stop point at 0 in preferences */
	clr_selorg_text();
	redraw_scr();
	outext();
}
	
void rand_dat_org(int orgIndex)
{
	int cnt=0, dir, xx, yy;
	
	xx = myrand() % WLSIZE;
	yy = myrand() % WLSIZE;
	while (world[xx][yy] < FOOD) {
		cnt++;
		if (cnt > 90)
			break;  // don't do an infinite loop
		xx = myrand() % WLSIZE;
		yy = myrand() % WLSIZE;
	}
	orgs[orgIndex].x = xx;
	orgs[orgIndex].y = yy;
	dir = myrand() % 4;
	orgs[orgIndex].direc = dir;
}

	 
int find_org(int x, int y)
{
	// finds index of last org with x,y position
	int i, index;
	
	index = -1;  // not found
	for (i = norgs; i >= 0; i--)
		if (orgs[i].x == x && orgs[i].y == y) {
			index = i;
			break;
		}
	return index;
}

/*void free_world(void)
{
	int i, j;
	
	for (i = 0; i < WLSIZE; i++)
		free(world[i]);
	free(world);
}*/

int myrand(void)
{
	// same as rand() but has access to seed
	seed = seed * 1103515245 + 12345;
	return((seed >> 16) & 0x7FFF);
}

void wait_mess(void)
/* waits for a few seconds so that error message */
/* from init_globals() can be seen. */
{
	clock_t lastime,currtime;
	double clocks_per_sec = (double) CLOCKS_PER_SEC;
	
	lastime = clock();
 	currtime = lastime;
 	while ((currtime-lastime) / clocks_per_sec < 4.0) 
		currtime = clock();
}

void waitime(void)
/* waits for delay seconds */
{
	clock_t lastime, currtime;
	double clocks_per_sec = (double) CLOCKS_PER_SEC;
	
	lastime = clock();
 	currtime = lastime;
 	while ((currtime-lastime) / clocks_per_sec < delay) 
		currtime = clock();
}


